from typing import Type, Callable, List, Union, Tuple

from bolinette import core, blnt, types


def model(model_name: str):
    def decorator(model_cls: Type['blnt.Model']):
        model_cls.__blnt__ = blnt.ModelMetadata(model_name)
        core.cache.models[model_name] = model_cls
        return model_cls
    return decorator


def model_property(function):
    return blnt.ModelProperty(function.__name__, function)


def mixin(mixin_name: str):
    def decorator(mixin_cls: Type['blnt.Mixin']):
        core.cache.mixins[mixin_name] = mixin_cls
        return mixin_cls
    return decorator


def with_mixin(mixin_name: str):
    def decorator(model_cls):
        mixin_cls = core.cache.mixins.get(mixin_name)
        for col_name, col_def in mixin_cls.columns().items():
            setattr(model_cls, col_name, col_def)
        for rel_name, rel_def in mixin_cls.relationships(model_cls).items():
            setattr(model_cls, rel_name, rel_def)
        return model_cls
    return decorator


def init_func(func: Callable[[core.BolinetteContext], None]):
    core.cache.init_funcs.append(func)
    return func


def seeder(func):
    core.cache.seeders.append(func)
    return func


def service(service_name: str, *, model_name: str = None):
    def decorator(service_cls: Type[Union['blnt.Service', 'blnt.SimpleService']]):
        service_cls.__blnt__ = blnt.ServiceMetadata(service_name, model_name or service_name)
        core.cache.services[service_name] = service_cls
        return service_cls
    return decorator


def controller(controller_name: str, path: str, *, service_name: str = None):
    def decorator(controller_cls: Type[Union['blnt.Controller', 'blnt.SimpleController']]):
        controller_cls.__blnt__ = blnt.ControllerMetadata(controller_name, path, service_name or controller_name)
        core.cache.controllers[controller_name] = controller_cls
        return controller_cls
    return decorator


def route(path: str, *, method: types.web.HttpMethod, access: 'types.web.AccessToken' = None,
          expects: Union[str, Tuple[str, str]] = None, returns: Union[str, Tuple[str, str]] = None,
          roles: List[str] = None):
    if expects is not None:
        if isinstance(expects, tuple):
            expects = blnt.ControllerExcepts(expects[0], expects[1] if len(expects) > 1 else 'default',
                                             patch='patch' in expects[2:])
        elif isinstance(expects, str):
            expects = blnt.ControllerExcepts(expects)
    if returns is not None:
        if isinstance(returns, tuple):
            returns = blnt.ControllerReturns(returns[0], returns[1] if len(returns) > 1 else 'default',
                                             as_list='as_list' in returns[2:], skip_none='skip_none' in returns[2:])
        elif isinstance(returns, str):
            returns = blnt.ControllerReturns(returns)

    def decorator(route_function: Callable):
        return blnt.ControllerRoute(route_function, path, method, access, expects, returns, roles)
    return decorator


def get(path: str, *, access: 'types.web.AccessToken' = None,
        expects: Union[str, Tuple[str, str], Tuple[str, str, str]] = None,
        returns: Union[str, Tuple[str, str]] = None,
        roles: List[str] = None):
    return route(path, method=types.web.HttpMethod.GET, access=access, expects=expects, returns=returns, roles=roles)


def post(path: str, *, access: 'types.web.AccessToken' = None,
         expects: Union[str, Tuple[str, str], Tuple[str, str, str]] = None,
         returns: Union[str, Tuple[str, str]] = None,
         roles: List[str] = None):
    return route(path, method=types.web.HttpMethod.POST, access=access, expects=expects, returns=returns, roles=roles)


def put(path: str, *, access: 'types.web.AccessToken' = None,
        expects: Union[str, Tuple[str, str], Tuple[str, str, str]] = None,
        returns: Union[str, Tuple[str, str]] = None,
        roles: List[str] = None):
    return route(path, method=types.web.HttpMethod.PUT, access=access, expects=expects, returns=returns, roles=roles)


def patch(path: str, *, access: 'types.web.AccessToken' = None,
          expects: Union[str, Tuple[str, str], Tuple[str, str, str]] = None,
          returns: Union[str, Tuple[str, str]] = None,
          roles: List[str] = None):
    return route(path, method=types.web.HttpMethod.PATCH, access=access, expects=expects, returns=returns, roles=roles)


def delete(path: str, *, access=None, expects=None, returns=None, roles: List[str] = None):
    return route(path, method=types.web.HttpMethod.DELETE, access=access, expects=expects, returns=returns, roles=roles)


def topic(topic_name: str):
    def decorator(topic_cls: Type['blnt.Topic']):
        topic_cls.__blnt__ = blnt.TopicMetadata(topic_name)
        core.cache.topics[topic_name] = topic_cls
        return topic_cls
    return decorator


def channel(rule: str):
    def decorator(channel_function: Callable):
        return blnt.TopicChannel(channel_function, rule)
    return decorator



import datetime
import random
import string
from types import SimpleNamespace

from bolinette import core, types


class Mocked:
    def __init__(self, name, context: core.BolinetteContext):
        self.name = name
        self.context = context
        self.fields = SimpleNamespace()

    @staticmethod
    def insert_entity(context: core.BolinetteContext, name, params):
        entity = context.table(name)(**params)
        context.db.session.add(entity)
        return entity

    def insert(self):
        fields = {}
        for key, value in self.fields.__dict__.items():
            fields[key] = value
        return self.insert_entity(self.context, self.name, fields)

    def to_response(self, key='default') -> dict:
        definition = self.context.mapping.response(self.name, key)
        return self.context.mapping.marshall(definition, self.fields, use_foreign_key=True)

    def to_payload(self, key='default') -> dict:
        definition = self.context.mapping.payload(self.name, key)
        return self.context.mapping.marshall(definition, self.fields, use_foreign_key=True)


class Mock:
    def __init__(self, context: core.BolinetteContext):
        self.context = context

    def _random_lower(self, rng, length):
        return ''.join(rng.choices(string.ascii_lowercase, k=length))

    def _random_symbols(self, rng, length):
        return ''.join(rng.choices(string.punctuation, k=length))

    def _random_int(self, rng, a, b):
        return rng.randint(a, b)

    def _random_decimal(self, rng, a, b):
        return rng.uniform(a, b)

    def _random_date(self, rng, start_date, end_date):
        time_between_dates = end_date - start_date
        days_between_dates = time_between_dates.days
        random_number_of_days = rng.randrange(days_between_dates)
        return start_date + datetime.timedelta(days=random_number_of_days)

    def __call__(self, m_id, model_name, *, post_mock_fn=None):
        columns = self.context.model(model_name).__props__.get_columns()
        rng = random.Random(hash(f'{model_name}.{m_id}'))
        mocked = Mocked(model_name, self.context)
        for _, column in columns.items():
            if column.primary_key:
                continue
            col_type = column.type
            if col_type == types.db.String:
                setattr(mocked.fields, column.name, self._random_lower(rng, 15))
            if col_type == types.db.Email:
                setattr(mocked.fields, column.name, f'{self._random_lower(rng, 10)}@{self._random_lower(rng, 5)}.com')
            if col_type == types.db.Password:
                setattr(mocked.fields, column.name, (self._random_lower(rng, 10) + str(self._random_int(rng, 1, 100))
                                                     + self._random_symbols(rng, 1)))
            if col_type == types.db.Integer:
                setattr(mocked.fields, column.name, self._random_int(rng, 1, 100))
            if col_type == types.db.Float:
                setattr(mocked.fields, column.name, self._random_int(rng, 1, 100))
            if col_type == types.db.Date:
                setattr(mocked.fields, column.name, self._random_date(rng, datetime.datetime(1900, 1, 1),
                                                                      datetime.datetime(2000, 1, 1)))
        if post_mock_fn and callable(post_mock_fn):
            post_mock_fn(mocked)
        return mocked



from typing import Dict, Any, Optional

from aiohttp import web as aio_web
from bolinette_common import paths

from bolinette import core, blnt


class BolinetteContext:
    def __init__(self, origin: str, app: Optional[aio_web.Application], *, profile=None, overrides=None):
        self.cwd = paths.cwd()
        self.origin = origin
        if app is not None:
            self.app = app
            self.env = core.Environment(self, profile=profile, overrides=overrides)
            self.db = core.DatabaseEngine(self)
            self.jwt = core.JWT(self)
            self.resources = core.BolinetteResources(self)
            self.sockets = core.BolinetteSockets(self)
            self.mapping = core.Mapping(self)
            self._tables: Dict[str, Any] = {}
            self._models: Dict[str, 'blnt.Model'] = {}
            self._repos: Dict[str, 'blnt.Repository'] = {}
            self._services: Dict[str, 'blnt.Service'] = {}
            self._controllers: Dict[str, 'blnt.Controller'] = {}
            self._ctx = {}

    def __getitem__(self, key):
        return self._ctx[key]

    def __setitem__(self, key, value):
        self._ctx[key] = value

    def model(self, name) -> Any:
        return self._models[name]

    def add_model(self, name, model: 'blnt.Model'):
        self._models[name] = model

    def instance_path(self, *path):
        return self.root_path('instance', *path)

    def env_path(self, *path):
        return self.root_path('env', *path)

    def static_path(self, *path):
        return self.root_path('static', *path)

    def root_path(self, *path):
        return paths.join(self.cwd, *path)

    def internal_path(self, *path):
        return paths.join(self.origin, *path)

    @property
    def models(self):
        return iter(self._models.items())

    def table(self, name) -> Any:
        return self._tables[name]

    def add_table(self, name, table):
        self._tables[name] = table

    @property
    def tables(self):
        return iter(self._tables.items())

    def repo(self, name) -> Any:
        return self._repos[name]

    def add_repo(self, name, repo: 'blnt.Repository'):
        self._repos[name] = repo

    @property
    def repos(self):
        return iter(self._repos.items())

    def service(self, name) -> Any:
        return self._services[name]

    def add_service(self, name, service: 'blnt.Service'):
        self._services[name] = service

    @property
    def services(self):
        return iter(self._services.items())

    def controller(self, name) -> Any:
        return self._controllers[name]

    def add_controller(self, name, controller: 'blnt.Controller'):
        self._controllers[name] = controller

    @property
    def controllers(self):
        return iter(self._repos.items())



from bolinette import core, blnt
from bolinette.exceptions import ParamConflictError


class Repository:
    def __init__(self, name: str, model: blnt.Model, context: 'core.BolinetteContext'):
        self.name = name
        self.model = model
        self.table = context.table(name)
        self.db = context.db

    def __repr__(self):
        return f'<Repository {self.name}>'

    @property
    def query(self):
        return self.db.session.query(self.table)

    def column(self, name: str):
        return getattr(self.table, name)

    async def get(self, identifier):
        return self.query.get(identifier)

    async def get_by(self, key, value):
        return self.query.filter_by(**{key: value}).all()

    async def get_first_by(self, key, value):
        return self.query.filter_by(**{key: value}).first()

    async def get_by_criteria(self, criteria):
        return self.query.filter(criteria).all()

    async def create(self, values):
        filtered = self._validate_model(values)
        entity = self.table(**filtered)
        self.db.session.add(entity)
        return entity

    async def update(self, entity, values):
        self._map_model(entity, values)
        return entity

    async def patch(self, entity, values):
        self._map_model(entity, values, patch=True)
        return entity

    async def delete(self, entity):
        self.db.session.delete(entity)
        return entity

    def _validate_model(self, values: dict):
        errors = []
        for column in self.model.__props__.get_columns().values():
            key = column.name
            if column.primary_key:
                continue
            value = values.get(key, None)
            if column.unique and value is not None:
                if self.query.filter(self.column(key) == value).first() is not None:
                    errors.append((key, value))
        if len(errors) > 0:
            raise ParamConflictError(params=errors)
        return values

    def _map_model(self, entity, values, patch=False):
        errors = []
        for _, column in self.model.__props__.get_columns().items():
            key = column.name
            if column.primary_key or (key not in values and patch):
                continue
            original = getattr(entity, key)
            new = values.get(key, None)
            if original == new:
                continue
            if column.unique and new is not None:
                if self.query.filter(self.column(key) == new).first() is not None:
                    errors.append((key, new))
            setattr(entity, key, new)
        if len(errors) > 0:
            raise ParamConflictError(params=errors)



import os
from typing import Dict, Any

import yaml
from bolinette_common import paths, console

from bolinette import core
from bolinette.exceptions import InitError


class Settings:
    def __init__(self):
        self._settings: Dict[str, Any] = {}

    def __getitem__(self, key: str):
        return self._settings.get(key.lower(), None)

    def __contains__(self, key: str):
        return key.lower() in self._settings

    def get(self, key: str, default=None):
        item = self[key]
        return item if item is not None else default

    def _cwd_path(self, *path):
        return paths.join(paths.cwd(), *path)

    def _reset(self, settings: Dict[str, Any]):
        self._settings = {}
        for key, value in settings.items():
            self._settings[key.lower()] = value

    def _load_from_file(self, file_name):
        try:
            with open(self._cwd_path('env', file_name), 'r') as f:
                d = yaml.safe_load(f)
                if not isinstance(d, dict):
                    raise InitError(inner=TypeError(f"'{file_name}' does not contain valid YAML syntax"))
                return d
        except FileNotFoundError:
            return {}


class InitSettings(Settings):
    def __init__(self):
        super().__init__()
        self._DEFAULT_PROFILE = 'development'
        self._init_checks()
        self.profile = self._read_profile()
        self._reset(self._load_from_file('init.yaml'))

    def _read_profile(self):
        try:
            with open(self._cwd_path('env', '.profile')) as f:
                profile = f.read().split('\n')[0].strip()
                if not profile:
                    console.error(f"Warning: empty 'env/.profile', defaulting to '{self._DEFAULT_PROFILE}'")
                    return self._DEFAULT_PROFILE
                return profile
        except FileNotFoundError:
            console.error(f"Warning: no 'env/.profile' file, defaulting to '{self._DEFAULT_PROFILE}'")
            return self._DEFAULT_PROFILE

    def _init_checks(self):
        dirs = [('env',), ('instance',)]
        for d in dirs:
            if not paths.exists(self._cwd_path(*d)):
                console.error(f"Warning: no '{'/'.join(d)}' dir, creating one")
                paths.mkdir(self._cwd_path(*d))


init = InitSettings()


class Environment(Settings):
    def __init__(self, context: 'core.BolinetteContext', *, profile=None, overrides=None):
        super().__init__()
        self.context = context
        profile = profile or init.profile or 'development'
        self._reset(self.merge_env_stack([
            self.default_env,
            self._load_from_file(f'env.{profile}.yaml'),
            self._load_from_file(f'env.local.{profile}.yaml'),
            self.load_from_os(),
            overrides or {},
            {'profile': profile}
        ]))

    @property
    def default_env(self):
        return {
            'APP_NAME': 'DEFAULT_NAME',
            'APP_DESC': 'DEFAULT_DESCRIPTION',
            'APP_VERSION': '0.0.1',
            'DBMS': 'SQLITE',
            'DEBUG': True,
            'HOST': '127.0.0.1',
            'PORT': '5000',
            'WEBAPP_FOLDER': self.context.root_path('webapp', 'dist')
        }

    def load_from_os(self):
        keys = {}
        for key in os.environ:
            if key.startswith('BLNT_'):
                keys[key[5:]] = os.environ[key]
        return keys

    def merge_env_stack(self, stack):
        settings = {}
        for source in stack:
            for key, value in source.items():
                settings[key.lower()] = value
        return settings



from enum import unique, Enum, auto


@unique
class HttpMethod(Enum):
    GET = auto()
    POST = auto()
    PUT = auto()
    PATCH = auto()
    DELETE = auto()

    @property
    def http_verb(self):
        if self.value == HttpMethod.GET.value:
            return 'GET'
        elif self.value == HttpMethod.POST.value:
            return 'POST'
        elif self.value == HttpMethod.PUT.value:
            return 'PUT'
        elif self.value == HttpMethod.PATCH.value:
            return 'PATCH'
        elif self.value == HttpMethod.DELETE.value:
            return 'DELETE'


@unique
class AccessToken(Enum):
    All = auto()
    Optional = auto()
    Required = auto()
    Fresh = auto()
    Refresh = auto()

    def check(self, context, request):
        _functions = {
            AccessToken.All.value: lambda _: None,
            AccessToken.Optional.value: lambda req: context.jwt.verify(req, optional=True),
            AccessToken.Required.value: lambda req: context.jwt.verify(req),
            AccessToken.Fresh.value: lambda req: context.jwt.verify(req, fresh=True),
            AccessToken.Refresh.value: lambda req: context.jwt.verify(req)
        }
        return _functions[self.value](request)



from bolinette.exceptions import (APIError, UnauthorizedError, BadRequestError, ConflictError, ForbiddenError,
                                  NotFoundError)


class Cookie:
    def __init__(self, name, value, *, path, http_only=True, expires=None, delete=False):
        self.name = name
        self.value = value
        self.expires = expires
        self.path = path
        self.http_only = http_only
        self.delete = delete


class APIResponse:
    def __init__(self, content, code):
        self.content = content
        self.code = code
        self.cookies = []


class Response:
    def __init__(self):
        self._exceptions = {
            UnauthorizedError: self.unauthorized,
            BadRequestError: self.bad_request,
            ConflictError: self.conflict,
            NotFoundError: self.not_found,
            ForbiddenError: self.forbidden,
            APIError: self.internal_server_error
        }

    def build_message(self, code, status, messages=None, data=None):
        if messages is None:
            messages = []
        if not isinstance(messages, list):
            messages = [messages]
        body = {
            'code': code,
            'status': status,
            'messages': messages
        }
        if data is not None:
            body['data'] = data
        return APIResponse(body, code)

    def ok(self, messages=None, data=None):
        return self.build_message(200, 'OK', messages, data)

    def created(self, messages=None, data=None):
        return self.build_message(201, 'CREATED', messages, data)

    def bad_request(self, messages=None, data=None):
        return self.build_message(400, 'BAD REQUEST', messages, data)

    def unauthorized(self, messages=None, data=None):
        return self.build_message(401, 'UNAUTHORIZED', messages, data)

    def forbidden(self, messages=None, data=None):
        return self.build_message(403, 'FORBIDDEN', messages, data)

    def not_found(self, messages=None, data=None):
        return self.build_message(404, 'NOT FOUND', messages, data)

    def method_not_allowed(self, messages=None, data=None):
        return self.build_message(405, 'METHOD NOT ALLOWED', messages, data)

    def conflict(self, messages=None, data=None):
        return self.build_message(409, 'CONFLICT', messages, data)

    def internal_server_error(self, messages=None, data=None):
        return self.build_message(500, 'INTERNAL SERVER ERROR', messages, data)

    def from_exception(self, exception: APIError):
        for except_cls in self._exceptions:
            if isinstance(exception, except_cls):
                return self._exceptions[except_cls](exception.messages)


response = Response()



class APIError(Exception):
    def __init__(self, name, messages):
        super().__init__(name)
        if not isinstance(messages, list):
            messages = [messages]
        self.messages = messages

    def __str__(self):
        return ", ".join(self.messages)


class InternalError(APIError):
    def __init__(self, messages, *, name='InternalError'):
        super().__init__(name, messages)


class NotFoundError(APIError):
    def __init__(self, messages, *, name='NotFoundError'):
        super().__init__(name, messages)


class ConflictError(APIError):
    def __init__(self, messages, *, name='ConflictError'):
        super().__init__(name, messages)


class BadRequestError(APIError):
    def __init__(self, messages, *, name='BadRequestError'):
        super().__init__(name, messages)


class ForbiddenError(APIError):
    def __init__(self, messages, *, name='ForbiddenError'):
        super().__init__(name, messages)


class UnauthorizedError(APIError):
    def __init__(self, messages, *, name='UnauthorizedError'):
        super().__init__(name, messages)


class EntityNotFoundError(NotFoundError):
    def __init__(self, **kwargs):
        params = kwargs.get('params', None)
        model = kwargs.get('model', None)
        key = kwargs.get('key', None)
        value = kwargs.get('value', None)
        if params is None:
            params = [(model, key, value)]
        messages = [f'{m}.not_found:{k}:{v}' for m, k, v in params]
        super().__init__(messages, name='EntityNotFoundError')


class ParamMissingError(BadRequestError):
    def __init__(self, **kwargs):
        params = kwargs.get('params', None)
        key = kwargs.get('key', None)
        if params is None:
            params = [key]
        messages = [f'param.required:{k}' for k in params]
        super().__init__(messages, name='ParamMissingError')


class ParamConflictError(ConflictError):
    def __init__(self, **kwargs):
        params = kwargs.get('params', None)
        key = kwargs.get('key', None)
        value = kwargs.get('value', None)
        if params is None:
            params = [(key, value)]
        messages = [f'param.conflict:{k}:{v}' for k, v in params]
        super().__init__(messages, name='ParamConflictError')


class InitError(Exception):
    def __init__(self, message: str = None, *, inner: Exception = None):
        self.message = message
        self.inner = inner

    def __str__(self):
        if self.message is None:
            return str(self.inner)
        return self.message



import sqlalchemy
from sqlalchemy import orm as sqlalchemy_orm

from bolinette import core, blnt
from bolinette.decorators import init_func


@init_func
def init_models(context: core.BolinetteContext):
    models = {}
    for model_name, model_cls in core.cache.models.items():
        models[model_name] = model_cls()
    orm_tables = {}
    orm_cols = {}
    for model_name, model in models.items():
        orm_cols[model_name] = {}
        for att_name, attribute in model.__props__.get_columns().items():
            attribute.name = att_name
            ref = None
            if attribute.reference:
                ref = sqlalchemy.ForeignKey(f'{attribute.reference.model_name}.{attribute.reference.column_name}')
            orm_cols[model_name][att_name] = sqlalchemy.Column(
                att_name, attribute.type.sqlalchemy_type, ref,
                primary_key=attribute.primary_key, nullable=attribute.nullable, unique=attribute.unique)
        orm_tables[model_name] = sqlalchemy.Table(model_name,
                                                  context.db.model.metadata,
                                                  *(orm_cols[model_name].values()))

    for model_name, model in models.items():
        orm_defs = {}
        for att_name, attribute in model.__props__.get_relationships().items():
            attribute.name = att_name
            backref = None
            if attribute.backref:
                backref = sqlalchemy_orm.backref(attribute.backref.key, lazy=attribute.backref.lazy)
            foreign_key = None
            if attribute.foreign_key:
                foreign_key = orm_cols[model_name][attribute.foreign_key.name]
            secondary = None
            if attribute.secondary:
                secondary = orm_tables[attribute.secondary]
            orm_defs[att_name] = sqlalchemy_orm.relationship(attribute.model_name, secondary=secondary,
                                                             lazy=attribute.lazy, foreign_keys=foreign_key,
                                                             backref=backref)

        orm_defs['__table__'] = orm_tables[model_name]
        orm_model = type(model_name, (context.db.model,), orm_defs)

        for att_name, attribute in model.__props__.get_properties().items():
            setattr(orm_model, att_name, property(attribute.function))

        context.add_model(model_name, model)
        context.add_table(model_name, orm_model)


@init_func
def init_repositories(context: core.BolinetteContext):
    for model_name, model in context.models:
        context.add_repo(model_name, blnt.Repository(model_name, model, context))


@init_func
def init_mappings(context: core.BolinetteContext):
    for model_name, model in context.models:
        context.mapping.register(model_name, model)


@init_func
def init_services(context: core.BolinetteContext):
    for service_name, service_cls in core.cache.services.items():
        context.add_service(service_name, service_cls(context))


@init_func
def init_controllers(context: core.BolinetteContext):
    for controller_name, controller_cls in core.cache.controllers.items():
        controller = controller_cls(context)
        base_path = '/api' if controller.__blnt__.api else ''
        for _, route in controller.__props__.get_routes().items():
            path = f'{base_path}{controller.__blnt__.path}{route.path}'
            context.resources.add_route(path, controller, route)
        if isinstance(controller, blnt.Controller):
            for route in controller.default_routes():
                path = f'{base_path}{controller.__blnt__.path}{route.path}'
                context.resources.add_route(path, controller, route)
        context.add_controller(controller_name, controller)


@init_func
def init_static_controller(context: core.BolinetteContext):
    controller = blnt.StaticController(context)
    for _, route in controller.__props__.get_routes().items():
        context.resources.add_route(route.path, controller, route)


@init_func
def init_topics(context: core.BolinetteContext):
    context.sockets.init_socket_handler()
    for topic_name, topic_cls in core.cache.topics.items():
        topic = topic_cls(context)
        context.sockets.add_topic(topic_name, topic)
        for channel_name, channel in topic.__props__.get_channels().items():
            context.sockets.add_channel(topic_name, channel)



from bolinette.blnt import functions
from bolinette.blnt.model import Model, ModelMetadata, ModelProperty
from bolinette.blnt.mixin import Mixin
from bolinette.blnt.repository import Repository
from bolinette.blnt.service import Service, ServiceMetadata, SimpleService, HistorizedService
from bolinette.blnt.controller import (Controller, SimpleController, ControllerMetadata, ControllerRoute,
                                       ControllerExcepts, ControllerReturns)
from bolinette.blnt.topic import Topic, TopicMetadata, TopicChannel
from bolinette.blnt.static import StaticController
